@@ -44,7 +44,6 @@
 #include <osgShadow/ShadowedScene>
 #include <osgShadow/LightSpacePerspectiveShadowMap>
 #include <osgShadow/ParallelSplitShadowMap>
-#include <osgShadow/ShadowVolume>
 //#include <osgShadow/SoftShadowMap>
 //#include <osgShadow/ShadowMap>
 
@@ -317,6 +316,7 @@ namespace mars {
       virtual void edit(const std::string &key, const std::string &value);
       void edit(unsigned long widgetID, const std::string &key,
                 const std::string &value);
+      osg::Vec3f getSelectedPos();
 
     private:
       mars::interfaces::GraphicData graphicOptions;
